#version 330 compatibility

#include "/lib/common.glsl"


in vec4 glcolor;
in vec3 viewPos; 
in vec3 worldSunDir;
in vec3 viewDir;
in vec3 sunDir;

uniform int renderStage;
uniform float viewHeight;
uniform float viewWidth;
uniform mat4 gbufferModelView;
uniform mat4 gbufferProjectionInverse;

vec3 sunCol = vec3(1.0, 0.3, 0.05);
 vec3 fogColor = vec3(0.5765, 0.9843, 1.0);
 vec3 skyColor = vec3(0.051, 0.2157, 0.9373);
 vec3 nightFogColor = vec3(0.0206, 0.0341, 0.0569);
 vec3 nightSkyColor = vec3(0.0019, 0.0039, 0.0157);
 vec3 earlyFogColor = vec3(0.7647, 0.3804, 0.1412);
 vec3 earlySkyColor = vec3(0.2157, 0.5922, 0.6588);
  vec3 lateFogColor = vec3(0.8784, 0.1647, 0.0863);
 vec3 lateSkyColor = vec3(0.5255, 0.9373, 0.902);
 vec3 MIE_Value = vec3(0.2549, 0.2549, 0.2549);

uniform int worldTime;


float skySmoothing(vec2 st, float pct)
{
    return  smoothstep( pct-0.02, pct, st.y) - smoothstep( pct, pct+0.02, st.y);
}

float fogify(float x, float w) {
	
    return w / (x * x  + w);
}



vec3 calcSkyColor(vec3 worldSunDir, vec3 dir) {
   vec3 horizonColor = vec3(0.0);
   vec3 zenithColor = vec3(0.0);
   
  
    if(worldTime >= 0 && worldTime < 1000)
    {
        float time = smoothstep(0, 1000, float(worldTime));
        horizonColor = mix(earlyFogColor, fogColor, time);
        zenithColor =  mix(earlySkyColor, skyColor, time);
    }
    else if(worldTime >= 1000 && worldTime < 10000)
    {
        float time = smoothstep(1000, 10000, float(worldTime));
        horizonColor = mix(fogColor, fogColor, time);
        zenithColor =  mix(skyColor, skyColor, time);
    }
    else if(worldTime >= 10000 && worldTime < 11500)
     {
        float time = smoothstep(10000, 11500, float(worldTime));
        horizonColor = mix(fogColor, earlyFogColor, time);
        zenithColor =  mix(skyColor, earlySkyColor, time);
    }
    else if(worldTime >= 11500 && worldTime < 13000)
     {
        float time = smoothstep(11500, 13000, float(worldTime));
        horizonColor = mix(earlyFogColor, nightFogColor, time);
        zenithColor =  mix(earlySkyColor, nightSkyColor, time);
    }
    else if(worldTime >= 13000 && worldTime < 23000)
     {
        float time = smoothstep(13000, 23000, float(worldTime));
        horizonColor = mix(nightFogColor, nightFogColor, time);
        zenithColor =  mix(nightSkyColor, nightSkyColor, time);
    }
    else if(worldTime >= 23000 && worldTime < 26000)
     {
        float time = smoothstep(23000, 26000, float(worldTime));
        horizonColor = mix(nightFogColor, lateFogColor, time);
        zenithColor =  mix(nightSkyColor, lateSkyColor, time);
    }

     float upDot = dot(dir, gbufferModelView[1].xyz); //not much, whats up with you?
        return mix(zenithColor, horizonColor, fogify(max(upDot, 0.01), 0.03));
    
}


vec3 screenToView(vec3 screenPos) {
    vec4 ndcPos = vec4(screenPos, 1.0) * 2.0 - 1.0;
    vec4 tmp = gbufferProjectionInverse * ndcPos;
    return tmp.xyz / tmp.w;
}

/* RENDERTARGETS: 0 */
layout(location = 0) out vec4 color;

void main() {
    if (renderStage == MC_RENDER_STAGE_STARS) {
        color = glcolor;
    } else {
        vec3 pos = screenToView(vec3(gl_FragCoord.xy / vec2(viewWidth, viewHeight), 1));
        color = vec4(calcSkyColor(worldSunDir, normalize(pos)), 1.0);
    }
}


